import json

from sp_api.api.product_fees.models.get_my_fees_estimate_response import GetMyFeesEstimateResponse
from sp_api.base.helpers import sp_endpoint, fill_query_params
from sp_api.base import Client, Marketplaces


class ProductFees(Client):
    def __init__(self, marketplace=Marketplaces.US, refresh_token=None):
        super().__init__(marketplace, refresh_token)

    @sp_endpoint('/products/fees/v0/listings/{}/feesEstimate', method='POST')
    def get_product_fees_estimate_for_sku(self, seller_sku, price: float, shipping_price=None, currency='USD',
                                          is_fba=False, **kwargs):
        kwargs.update(self._create_body(price, shipping_price, currency, is_fba))
        return GetMyFeesEstimateResponse(
            **self._request(fill_query_params(kwargs.pop('path'), seller_sku), data=kwargs).json())

    @sp_endpoint('/products/fees/v0/items/{}/feesEstimate', method='POST')
    def get_product_fees_estimate_for_asin(self, asin, price: float, currency='USD', shipping_price=None, is_fba=False,
                                           **kwargs):
        kwargs.update(self._create_body(price, shipping_price, currency, is_fba))
        return GetMyFeesEstimateResponse(
            **self._request(fill_query_params(kwargs.pop('path'), asin), data=kwargs).json())

    def _create_body(self, price, shipping_price=None, currency='USD', is_fba=False):
        return {
            'FeesEstimateRequest': {
                'Identifier': price,
                'PriceToEstimateFees': {
                    'ListingPrice': {
                        'Amount': price,
                        'CurrencyCode': currency
                    },
                    'Shipping': {
                        'Amount': shipping_price,
                        'CurrencyCode': currency
                    } if shipping_price else None,
                },
                'IsAmazonFulfilled': is_fba,
                'MarketplaceId': self.marketplace_id
            }
        }



# coding: utf-8

"""
    Selling Partner API for Reports

    The Selling Partner API for Reports lets you retrieve and manage a variety of reports that can help selling partners manage their businesses.  # noqa: E501

    OpenAPI spec version: 2020-09-04
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class Report(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'marketplace_ids': 'list[str]',
        'report_id': 'str',
        'report_type': 'str',
        'data_start_time': 'datetime',
        'data_end_time': 'datetime',
        'report_schedule_id': 'str',
        'created_time': 'datetime',
        'processing_status': 'str',
        'processing_start_time': 'datetime',
        'processing_end_time': 'datetime',
        'report_document_id': 'str'
    }

    attribute_map = {
        'marketplace_ids': 'marketplaceIds',
        'report_id': 'reportId',
        'report_type': 'reportType',
        'data_start_time': 'dataStartTime',
        'data_end_time': 'dataEndTime',
        'report_schedule_id': 'reportScheduleId',
        'created_time': 'createdTime',
        'processing_status': 'processingStatus',
        'processing_start_time': 'processingStartTime',
        'processing_end_time': 'processingEndTime',
        'report_document_id': 'reportDocumentId'
    }

    def __init__(self, marketplace_ids=None, report_id=None, report_type=None, data_start_time=None, data_end_time=None, report_schedule_id=None, created_time=None, processing_status=None, processing_start_time=None, processing_end_time=None, report_document_id=None):  # noqa: E501
        """Report - a model defined in Swagger"""  # noqa: E501
        self._marketplace_ids = None
        self._report_id = None
        self._report_type = None
        self._data_start_time = None
        self._data_end_time = None
        self._report_schedule_id = None
        self._created_time = None
        self._processing_status = None
        self._processing_start_time = None
        self._processing_end_time = None
        self._report_document_id = None
        self.discriminator = None
        if marketplace_ids is not None:
            self.marketplace_ids = marketplace_ids
        self.report_id = report_id
        self.report_type = report_type
        if data_start_time is not None:
            self.data_start_time = data_start_time
        if data_end_time is not None:
            self.data_end_time = data_end_time
        if report_schedule_id is not None:
            self.report_schedule_id = report_schedule_id
        self.created_time = created_time
        self.processing_status = processing_status
        if processing_start_time is not None:
            self.processing_start_time = processing_start_time
        if processing_end_time is not None:
            self.processing_end_time = processing_end_time
        if report_document_id is not None:
            self.report_document_id = report_document_id

    @property
    def marketplace_ids(self):
        """Gets the marketplace_ids of this Report.  # noqa: E501

        A list of marketplace identifiers for the report.  # noqa: E501

        :return: The marketplace_ids of this Report.  # noqa: E501
        :rtype: list[str]
        """
        return self._marketplace_ids

    @marketplace_ids.setter
    def marketplace_ids(self, marketplace_ids):
        """Sets the marketplace_ids of this Report.

        A list of marketplace identifiers for the report.  # noqa: E501

        :param marketplace_ids: The marketplace_ids of this Report.  # noqa: E501
        :type: list[str]
        """

        self._marketplace_ids = marketplace_ids

    @property
    def report_id(self):
        """Gets the report_id of this Report.  # noqa: E501

        The identifier for the report. This identifier is unique only in combination with a seller ID.  # noqa: E501

        :return: The report_id of this Report.  # noqa: E501
        :rtype: str
        """
        return self._report_id

    @report_id.setter
    def report_id(self, report_id):
        """Sets the report_id of this Report.

        The identifier for the report. This identifier is unique only in combination with a seller ID.  # noqa: E501

        :param report_id: The report_id of this Report.  # noqa: E501
        :type: str
        """
        if report_id is None:
            raise ValueError("Invalid value for `report_id`, must not be `None`")  # noqa: E501

        self._report_id = report_id

    @property
    def report_type(self):
        """Gets the report_type of this Report.  # noqa: E501

        The report type.  # noqa: E501

        :return: The report_type of this Report.  # noqa: E501
        :rtype: str
        """
        return self._report_type

    @report_type.setter
    def report_type(self, report_type):
        """Sets the report_type of this Report.

        The report type.  # noqa: E501

        :param report_type: The report_type of this Report.  # noqa: E501
        :type: str
        """
        if report_type is None:
            raise ValueError("Invalid value for `report_type`, must not be `None`")  # noqa: E501

        self._report_type = report_type

    @property
    def data_start_time(self):
        """Gets the data_start_time of this Report.  # noqa: E501

        The start of a date and time range used for selecting the data to report.  # noqa: E501

        :return: The data_start_time of this Report.  # noqa: E501
        :rtype: datetime
        """
        return self._data_start_time

    @data_start_time.setter
    def data_start_time(self, data_start_time):
        """Sets the data_start_time of this Report.

        The start of a date and time range used for selecting the data to report.  # noqa: E501

        :param data_start_time: The data_start_time of this Report.  # noqa: E501
        :type: datetime
        """

        self._data_start_time = data_start_time

    @property
    def data_end_time(self):
        """Gets the data_end_time of this Report.  # noqa: E501

        The end of a date and time range used for selecting the data to report.  # noqa: E501

        :return: The data_end_time of this Report.  # noqa: E501
        :rtype: datetime
        """
        return self._data_end_time

    @data_end_time.setter
    def data_end_time(self, data_end_time):
        """Sets the data_end_time of this Report.

        The end of a date and time range used for selecting the data to report.  # noqa: E501

        :param data_end_time: The data_end_time of this Report.  # noqa: E501
        :type: datetime
        """

        self._data_end_time = data_end_time

    @property
    def report_schedule_id(self):
        """Gets the report_schedule_id of this Report.  # noqa: E501

        The identifier of the report schedule that created this report (if any). This identifier is unique only in combination with a seller ID.  # noqa: E501

        :return: The report_schedule_id of this Report.  # noqa: E501
        :rtype: str
        """
        return self._report_schedule_id

    @report_schedule_id.setter
    def report_schedule_id(self, report_schedule_id):
        """Sets the report_schedule_id of this Report.

        The identifier of the report schedule that created this report (if any). This identifier is unique only in combination with a seller ID.  # noqa: E501

        :param report_schedule_id: The report_schedule_id of this Report.  # noqa: E501
        :type: str
        """

        self._report_schedule_id = report_schedule_id

    @property
    def created_time(self):
        """Gets the created_time of this Report.  # noqa: E501

        The date and time when the report was created.  # noqa: E501

        :return: The created_time of this Report.  # noqa: E501
        :rtype: datetime
        """
        return self._created_time

    @created_time.setter
    def created_time(self, created_time):
        """Sets the created_time of this Report.

        The date and time when the report was created.  # noqa: E501

        :param created_time: The created_time of this Report.  # noqa: E501
        :type: datetime
        """
        if created_time is None:
            raise ValueError("Invalid value for `created_time`, must not be `None`")  # noqa: E501

        self._created_time = created_time

    @property
    def processing_status(self):
        """Gets the processing_status of this Report.  # noqa: E501

        The processing status of the report.  # noqa: E501

        :return: The processing_status of this Report.  # noqa: E501
        :rtype: str
        """
        return self._processing_status

    @processing_status.setter
    def processing_status(self, processing_status):
        """Sets the processing_status of this Report.

        The processing status of the report.  # noqa: E501

        :param processing_status: The processing_status of this Report.  # noqa: E501
        :type: str
        """
        if processing_status is None:
            raise ValueError("Invalid value for `processing_status`, must not be `None`")  # noqa: E501
        allowed_values = ["CANCELLED", "DONE", "FATAL", "IN_PROGRESS", "IN_QUEUE"]  # noqa: E501
        if processing_status not in allowed_values:
            raise ValueError(
                "Invalid value for `processing_status` ({0}), must be one of {1}"  # noqa: E501
                .format(processing_status, allowed_values)
            )

        self._processing_status = processing_status

    @property
    def processing_start_time(self):
        """Gets the processing_start_time of this Report.  # noqa: E501

        The date and time when the report processing started, in ISO 8601 date time format.  # noqa: E501

        :return: The processing_start_time of this Report.  # noqa: E501
        :rtype: datetime
        """
        return self._processing_start_time

    @processing_start_time.setter
    def processing_start_time(self, processing_start_time):
        """Sets the processing_start_time of this Report.

        The date and time when the report processing started, in ISO 8601 date time format.  # noqa: E501

        :param processing_start_time: The processing_start_time of this Report.  # noqa: E501
        :type: datetime
        """

        self._processing_start_time = processing_start_time

    @property
    def processing_end_time(self):
        """Gets the processing_end_time of this Report.  # noqa: E501

        The date and time when the report processing completed, in ISO 8601 date time format.  # noqa: E501

        :return: The processing_end_time of this Report.  # noqa: E501
        :rtype: datetime
        """
        return self._processing_end_time

    @processing_end_time.setter
    def processing_end_time(self, processing_end_time):
        """Sets the processing_end_time of this Report.

        The date and time when the report processing completed, in ISO 8601 date time format.  # noqa: E501

        :param processing_end_time: The processing_end_time of this Report.  # noqa: E501
        :type: datetime
        """

        self._processing_end_time = processing_end_time

    @property
    def report_document_id(self):
        """Gets the report_document_id of this Report.  # noqa: E501

        The identifier for the report document. Pass this into the getReportDocument operation to get the information you will need to retrieve and decrypt the report document's contents.  # noqa: E501

        :return: The report_document_id of this Report.  # noqa: E501
        :rtype: str
        """
        return self._report_document_id

    @report_document_id.setter
    def report_document_id(self, report_document_id):
        """Sets the report_document_id of this Report.

        The identifier for the report document. Pass this into the getReportDocument operation to get the information you will need to retrieve and decrypt the report document's contents.  # noqa: E501

        :param report_document_id: The report_document_id of this Report.  # noqa: E501
        :type: str
        """

        self._report_document_id = report_document_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Report, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Report):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



import json
import os
import re
from datetime import datetime

import boto3
from cachetools import TTLCache
from requests import request

from sp_api.auth import AccessTokenClient, AccessTokenResponse
from .base_client import BaseClient
from .marketplaces import Marketplaces
from sp_api.base import AWSSigV4

role_cache = TTLCache(maxsize=10, ttl=3600)
client = boto3.client('sts',
                      aws_access_key_id=os.environ.get('SP_API_ACCESS_KEY'),
                      aws_secret_access_key=os.environ.get('SP_API_SECRET_KEY')
                      )


class SellingApiException(BaseException):
    pass


class Client(BaseClient):
    def __init__(self,
                 marketplace: Marketplaces,
                 refresh_token=None):
        self.endpoint = marketplace.endpoint
        self.marketplace_id = marketplace.marketplace_id
        self._auth = AccessTokenClient(refresh_token)

    @staticmethod
    def set_role():
        role = client.assume_role(
            RoleArn=os.environ.get('SP_API_ROLE_ARN'),
            RoleSessionName='guid'
        )
        role_cache['role'] = role
        return role

    @property
    def headers(self):
        return {
            'host': self.endpoint[8:],
            'user-agent': self.user_agent,
            'x-amz-access-token': self.auth.access_token,
            'x-amz-date': datetime.utcnow().strftime('%Y%m%dT%H%M%SZ'),
            'content-type': 'application/json'
        }

    @property
    def auth(self) -> AccessTokenResponse:
        return self._auth.get_auth()

    @property
    def grantless_auth(self) -> AccessTokenResponse:
        return self._auth.get_grantless_auth()

    @property
    def role(self):
        try:
            role = role_cache['role']
        except KeyError:
            role = self.set_role()
        return role.get('Credentials')

    def _sign_request(self):
        role = self.role
        return AWSSigV4('execute-api',
                        aws_access_key_id=role.get('AccessKeyId'),
                        aws_secret_access_key=role.get('SecretAccessKey'),
                        region=os.environ.get('SP_AWS_REGION', 'us-east-1'),
                        aws_session_token=role.get('SessionToken')
                        )

    def _request(self, path: str, *, data: dict = None, params: dict = None, headers=None):

        if params is None:
            params = {}
        if data is None:
            data = {}

        self.method = params.pop('method', data.pop('method', 'GET'))

        if self.method == 'POST':
            data.update({'marketplaceIds': [self.marketplace_id], 'MarketplaceIds': [self.marketplace_id]})
            data = json.dumps(data)
        else:
            if 'MarketplaceIds' not in params and 'marketplaceIds' not in params:
                params.update({'MarketplaceIds': self.marketplace_id, 'marketplaceIds': self.marketplace_id})

        res = request(self.method, self.endpoint + path, params=params, data=data, headers=headers or self.headers,
                      auth=self._sign_request())

        if e := res.json().get('errors', None):
            raise SellingApiException(e)
        return res

    def _request_grantless_operation(self, path: str, *, data: dict = None, params: dict = None):
        headers = {
            'host': self.endpoint[8:],
            'user-agent': self.user_agent,
            'x-amz-access-token': self.grantless_auth.access_token,
            'x-amz-date': datetime.utcnow().strftime('%Y%m%dT%H%M%SZ'),
            'content-type': 'application/json'
        }

        return self._request(path, data=data, params=params, headers=headers)



# coding: utf-8

"""
    Selling Partner API for Product Fees

    The Selling Partner API for Product Fees lets you programmatically retrieve estimated fees for a product. You can then account for those fees in your pricing.  # noqa: E501

    OpenAPI spec version: v0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class FeesEstimate(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'time_of_fees_estimation': 'datetime',
        'total_fees_estimate': 'MoneyType',
        'fee_detail_list': 'FeeDetailList'
    }

    attribute_map = {
        'time_of_fees_estimation': 'TimeOfFeesEstimation',
        'total_fees_estimate': 'TotalFeesEstimate',
        'fee_detail_list': 'FeeDetailList'
    }

    def __init__(self, time_of_fees_estimation=None, total_fees_estimate=None, fee_detail_list=None):  # noqa: E501
        """FeesEstimate - a model defined in Swagger"""  # noqa: E501
        self._time_of_fees_estimation = None
        self._total_fees_estimate = None
        self._fee_detail_list = None
        self.discriminator = None
        self.time_of_fees_estimation = time_of_fees_estimation
        if total_fees_estimate is not None:
            self.total_fees_estimate = total_fees_estimate
        if fee_detail_list is not None:
            self.fee_detail_list = fee_detail_list

    @property
    def time_of_fees_estimation(self):
        """Gets the time_of_fees_estimation of this FeesEstimate.  # noqa: E501

        The time for which the fees were estimated. This defaults to the time the request is made.  # noqa: E501

        :return: The time_of_fees_estimation of this FeesEstimate.  # noqa: E501
        :rtype: datetime
        """
        return self._time_of_fees_estimation

    @time_of_fees_estimation.setter
    def time_of_fees_estimation(self, time_of_fees_estimation):
        """Sets the time_of_fees_estimation of this FeesEstimate.

        The time for which the fees were estimated. This defaults to the time the request is made.  # noqa: E501

        :param time_of_fees_estimation: The time_of_fees_estimation of this FeesEstimate.  # noqa: E501
        :type: datetime
        """
        if time_of_fees_estimation is None:
            raise ValueError("Invalid value for `time_of_fees_estimation`, must not be `None`")  # noqa: E501

        self._time_of_fees_estimation = time_of_fees_estimation

    @property
    def total_fees_estimate(self):
        """Gets the total_fees_estimate of this FeesEstimate.  # noqa: E501


        :return: The total_fees_estimate of this FeesEstimate.  # noqa: E501
        :rtype: MoneyType
        """
        return self._total_fees_estimate

    @total_fees_estimate.setter
    def total_fees_estimate(self, total_fees_estimate):
        """Sets the total_fees_estimate of this FeesEstimate.


        :param total_fees_estimate: The total_fees_estimate of this FeesEstimate.  # noqa: E501
        :type: MoneyType
        """

        self._total_fees_estimate = total_fees_estimate

    @property
    def fee_detail_list(self):
        """Gets the fee_detail_list of this FeesEstimate.  # noqa: E501


        :return: The fee_detail_list of this FeesEstimate.  # noqa: E501
        :rtype: FeeDetailList
        """
        return self._fee_detail_list

    @fee_detail_list.setter
    def fee_detail_list(self, fee_detail_list):
        """Sets the fee_detail_list of this FeesEstimate.


        :param fee_detail_list: The fee_detail_list of this FeesEstimate.  # noqa: E501
        :type: FeeDetailList
        """

        self._fee_detail_list = fee_detail_list

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(FeesEstimate, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FeesEstimate):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from sp_api.base import sp_endpoint, fill_query_params
from sp_api.api.orders.models.get_order_items_response import GetOrderItemsResponse
from sp_api.api.orders.models.get_order_response import GetOrderResponse
from sp_api.api.orders.models.get_orders_response import GetOrdersResponse
from sp_api.base import Client, Marketplaces


class Orders(Client):
    def __init__(self, marketplace=Marketplaces.US, refresh_token=None):
        super().__init__(marketplace, refresh_token)

    @sp_endpoint('/orders/v0/orders')
    def get_orders(self, **kwargs):
        """
        Returns orders created or updated during the time frame indicated by the specified parameters.
        You can also apply a range of filtering criteria to narrow the list of orders returned.
        If NextToken is present, that will be used to retrieve the orders instead of other criteria.

        **Usage Plan:**

        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 1 |

        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        :param kwargs:
        :return:
        """
        return GetOrdersResponse(
            **self._request(kwargs.pop('path'), params={**kwargs}).json())

    @sp_endpoint('/orders/v0/orders/{}')
    def get_order(self, order_id, **kwargs):
        """
        Returns the order indicated by the specified order ID.

        **Usage Plan:**

        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 1 |

        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        :param order_id:
        :param kwargs:
        :return:
        """
        return GetOrderResponse(
            **self._request(fill_query_params(kwargs.pop('path'), order_id),
                            params={**kwargs}).json())

    @sp_endpoint('/orders/v0/orders/{}/orderItems')
    def get_order_items(self, order_id, **kwargs):
        """
        Returns detailed order item information for the order indicated by the specified order ID.
        If NextToken is provided, it's used to retrieve the next page of order items.

        Note: When an order is in the Pending state (the order has been placed but payment has not been authorized),
        the getOrderItems operation does not return information about pricing, taxes, shipping charges, gift status or
        promotions for the order items in the order.
        After an order leaves the Pending state (this occurs when payment has been authorized) and enters the Unshipped,
        Partially Shipped, or Shipped state, the getOrderItems operation returns information about pricing, taxes,
        shipping charges, gift status and promotions for the order items in the order.

        **Usage Plan:**

        | Rate (requests per second) | Burst |
        | ---- | ---- |
        | 1 | 1 |

        For more information, see "Usage Plans and Rate Limits" in the Selling Partner API documentation.
        :param order_id:
        :param kwargs:
        :return:
        """
        return GetOrderItemsResponse(
            **self._request(fill_query_params(kwargs.pop('path'), order_id), params={**kwargs}).json())



from sp_api.api.finances.models.list_financial_events_response import ListFinancialEventsResponse
from sp_api.base import Client, Marketplaces
from sp_api.base import sp_endpoint, fill_query_params


class Finances(Client):
    def __init__(self, marketplace=Marketplaces.US, *, refresh_token=None):
        super().__init__(marketplace, refresh_token)

    @sp_endpoint('/finances/v0/orders/{}/financialEvents')
    def get_financial_events_for_order(self, order_id, **kwargs):
        return ListFinancialEventsResponse(
            **self._request(fill_query_params(kwargs.pop('path'), order_id), params={**kwargs}).json()
        )



# coding: utf-8

"""
    Selling Partner API for Pricing

    The Selling Partner API for Pricing helps you programmatically retrieve product pricing and offer information for Amazon Marketplace products.  # noqa: E501

    OpenAPI spec version: v0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class Product(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'identifiers': 'IdentifierType',
        'attribute_sets': 'AttributeSetList',
        'relationships': 'RelationshipList',
        'competitive_pricing': 'CompetitivePricingType',
        'sales_rankings': 'SalesRankList',
        'offers': 'OffersList'
    }

    attribute_map = {
        'identifiers': 'Identifiers',
        'attribute_sets': 'AttributeSets',
        'relationships': 'Relationships',
        'competitive_pricing': 'CompetitivePricing',
        'sales_rankings': 'SalesRankings',
        'offers': 'Offers'
    }

    def __init__(self, identifiers=None, attribute_sets=None, relationships=None, competitive_pricing=None, sales_rankings=None, offers=None):  # noqa: E501
        """Product - a model defined in Swagger"""  # noqa: E501
        self._identifiers = None
        self._attribute_sets = None
        self._relationships = None
        self._competitive_pricing = None
        self._sales_rankings = None
        self._offers = None
        self.discriminator = None
        self.identifiers = identifiers
        if attribute_sets is not None:
            self.attribute_sets = attribute_sets
        if relationships is not None:
            self.relationships = relationships
        if competitive_pricing is not None:
            self.competitive_pricing = competitive_pricing
        if sales_rankings is not None:
            self.sales_rankings = sales_rankings
        if offers is not None:
            self.offers = offers

    @property
    def identifiers(self):
        """Gets the identifiers of this Product.  # noqa: E501


        :return: The identifiers of this Product.  # noqa: E501
        :rtype: IdentifierType
        """
        return self._identifiers

    @identifiers.setter
    def identifiers(self, identifiers):
        """Sets the identifiers of this Product.


        :param identifiers: The identifiers of this Product.  # noqa: E501
        :type: IdentifierType
        """
        if identifiers is None:
            raise ValueError("Invalid value for `identifiers`, must not be `None`")  # noqa: E501

        self._identifiers = identifiers

    @property
    def attribute_sets(self):
        """Gets the attribute_sets of this Product.  # noqa: E501


        :return: The attribute_sets of this Product.  # noqa: E501
        :rtype: AttributeSetList
        """
        return self._attribute_sets

    @attribute_sets.setter
    def attribute_sets(self, attribute_sets):
        """Sets the attribute_sets of this Product.


        :param attribute_sets: The attribute_sets of this Product.  # noqa: E501
        :type: AttributeSetList
        """

        self._attribute_sets = attribute_sets

    @property
    def relationships(self):
        """Gets the relationships of this Product.  # noqa: E501


        :return: The relationships of this Product.  # noqa: E501
        :rtype: RelationshipList
        """
        return self._relationships

    @relationships.setter
    def relationships(self, relationships):
        """Sets the relationships of this Product.


        :param relationships: The relationships of this Product.  # noqa: E501
        :type: RelationshipList
        """

        self._relationships = relationships

    @property
    def competitive_pricing(self):
        """Gets the competitive_pricing of this Product.  # noqa: E501


        :return: The competitive_pricing of this Product.  # noqa: E501
        :rtype: CompetitivePricingType
        """
        return self._competitive_pricing

    @competitive_pricing.setter
    def competitive_pricing(self, competitive_pricing):
        """Sets the competitive_pricing of this Product.


        :param competitive_pricing: The competitive_pricing of this Product.  # noqa: E501
        :type: CompetitivePricingType
        """

        self._competitive_pricing = competitive_pricing

    @property
    def sales_rankings(self):
        """Gets the sales_rankings of this Product.  # noqa: E501


        :return: The sales_rankings of this Product.  # noqa: E501
        :rtype: SalesRankList
        """
        return self._sales_rankings

    @sales_rankings.setter
    def sales_rankings(self, sales_rankings):
        """Sets the sales_rankings of this Product.


        :param sales_rankings: The sales_rankings of this Product.  # noqa: E501
        :type: SalesRankList
        """

        self._sales_rankings = sales_rankings

    @property
    def offers(self):
        """Gets the offers of this Product.  # noqa: E501


        :return: The offers of this Product.  # noqa: E501
        :rtype: OffersList
        """
        return self._offers

    @offers.setter
    def offers(self, offers):
        """Sets the offers of this Product.


        :param offers: The offers of this Product.  # noqa: E501
        :type: OffersList
        """

        self._offers = offers

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Product, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Product):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Selling Partner API for Sales

    The Selling Partner API for Sales provides APIs related to sales performance.  # noqa: E501

    OpenAPI spec version: v1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class OrderMetricsInterval(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'interval': 'str',
        'unit_count': 'int',
        'order_item_count': 'int',
        'order_count': 'int',
        'average_unit_price': 'Money',
        'total_sales': 'Money'
    }

    attribute_map = {
        'interval': 'interval',
        'unit_count': 'unitCount',
        'order_item_count': 'orderItemCount',
        'order_count': 'orderCount',
        'average_unit_price': 'averageUnitPrice',
        'total_sales': 'totalSales'
    }

    def __init__(self, interval=None, unit_count=None, order_item_count=None, order_count=None, average_unit_price=None, total_sales=None):  # noqa: E501
        """OrderMetricsInterval - a model defined in Swagger"""  # noqa: E501
        self._interval = None
        self._unit_count = None
        self._order_item_count = None
        self._order_count = None
        self._average_unit_price = None
        self._total_sales = None
        self.discriminator = None
        self.interval = interval
        self.unit_count = unit_count
        self.order_item_count = order_item_count
        self.order_count = order_count
        self.average_unit_price = average_unit_price
        self.total_sales = total_sales

    @property
    def interval(self):
        """Gets the interval of this OrderMetricsInterval.  # noqa: E501

        The interval of time based on requested granularity (ex. Hour, Day, etc.) If this is the first or the last interval from the list, it might contain incomplete data if the requested interval doesn't align with the requested granularity (ex. request interval 2018-09-01T02:00:00Z--2018-09-04T19:00:00Z and granularity day will result in Sept 1st UTC day and Sept 4th UTC days having partial data).  # noqa: E501

        :return: The interval of this OrderMetricsInterval.  # noqa: E501
        :rtype: str
        """
        return self._interval

    @interval.setter
    def interval(self, interval):
        """Sets the interval of this OrderMetricsInterval.

        The interval of time based on requested granularity (ex. Hour, Day, etc.) If this is the first or the last interval from the list, it might contain incomplete data if the requested interval doesn't align with the requested granularity (ex. request interval 2018-09-01T02:00:00Z--2018-09-04T19:00:00Z and granularity day will result in Sept 1st UTC day and Sept 4th UTC days having partial data).  # noqa: E501

        :param interval: The interval of this OrderMetricsInterval.  # noqa: E501
        :type: str
        """
        if interval is None:
            raise ValueError("Invalid value for `interval`, must not be `None`")  # noqa: E501

        self._interval = interval

    @property
    def unit_count(self):
        """Gets the unit_count of this OrderMetricsInterval.  # noqa: E501

        The number of units in orders based on the specified filters.  # noqa: E501

        :return: The unit_count of this OrderMetricsInterval.  # noqa: E501
        :rtype: int
        """
        return self._unit_count

    @unit_count.setter
    def unit_count(self, unit_count):
        """Sets the unit_count of this OrderMetricsInterval.

        The number of units in orders based on the specified filters.  # noqa: E501

        :param unit_count: The unit_count of this OrderMetricsInterval.  # noqa: E501
        :type: int
        """
        if unit_count is None:
            raise ValueError("Invalid value for `unit_count`, must not be `None`")  # noqa: E501

        self._unit_count = unit_count

    @property
    def order_item_count(self):
        """Gets the order_item_count of this OrderMetricsInterval.  # noqa: E501

        The number of order items based on the specified filters.  # noqa: E501

        :return: The order_item_count of this OrderMetricsInterval.  # noqa: E501
        :rtype: int
        """
        return self._order_item_count

    @order_item_count.setter
    def order_item_count(self, order_item_count):
        """Sets the order_item_count of this OrderMetricsInterval.

        The number of order items based on the specified filters.  # noqa: E501

        :param order_item_count: The order_item_count of this OrderMetricsInterval.  # noqa: E501
        :type: int
        """
        if order_item_count is None:
            raise ValueError("Invalid value for `order_item_count`, must not be `None`")  # noqa: E501

        self._order_item_count = order_item_count

    @property
    def order_count(self):
        """Gets the order_count of this OrderMetricsInterval.  # noqa: E501

        The number of orders based on the specified filters.  # noqa: E501

        :return: The order_count of this OrderMetricsInterval.  # noqa: E501
        :rtype: int
        """
        return self._order_count

    @order_count.setter
    def order_count(self, order_count):
        """Sets the order_count of this OrderMetricsInterval.

        The number of orders based on the specified filters.  # noqa: E501

        :param order_count: The order_count of this OrderMetricsInterval.  # noqa: E501
        :type: int
        """
        if order_count is None:
            raise ValueError("Invalid value for `order_count`, must not be `None`")  # noqa: E501

        self._order_count = order_count

    @property
    def average_unit_price(self):
        """Gets the average_unit_price of this OrderMetricsInterval.  # noqa: E501


        :return: The average_unit_price of this OrderMetricsInterval.  # noqa: E501
        :rtype: Money
        """
        return self._average_unit_price

    @average_unit_price.setter
    def average_unit_price(self, average_unit_price):
        """Sets the average_unit_price of this OrderMetricsInterval.


        :param average_unit_price: The average_unit_price of this OrderMetricsInterval.  # noqa: E501
        :type: Money
        """
        if average_unit_price is None:
            raise ValueError("Invalid value for `average_unit_price`, must not be `None`")  # noqa: E501

        self._average_unit_price = average_unit_price

    @property
    def total_sales(self):
        """Gets the total_sales of this OrderMetricsInterval.  # noqa: E501


        :return: The total_sales of this OrderMetricsInterval.  # noqa: E501
        :rtype: Money
        """
        return self._total_sales

    @total_sales.setter
    def total_sales(self, total_sales):
        """Sets the total_sales of this OrderMetricsInterval.


        :param total_sales: The total_sales of this OrderMetricsInterval.  # noqa: E501
        :type: Money
        """
        if total_sales is None:
            raise ValueError("Invalid value for `total_sales`, must not be `None`")  # noqa: E501

        self._total_sales = total_sales

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(OrderMetricsInterval, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrderMetricsInterval):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Selling Partner API for Orders

    The Selling Partner API for Orders helps you programmatically retrieve order information. These APIs let you develop fast, flexible, custom applications in areas like order synchronization, order research, and demand-based decision support tools.  # noqa: E501

    OpenAPI spec version: v0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

from api.orders.models.order_list import OrderList


class OrdersList(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'orders': 'OrderList',
        'next_token': 'str',
        'last_updated_before': 'str',
        'created_before': 'str'
    }

    attribute_map = {
        'orders': 'Orders',
        'next_token': 'NextToken',
        'last_updated_before': 'LastUpdatedBefore',
        'created_before': 'CreatedBefore'
    }

    def __init__(self, orders=None, next_token=None, last_updated_before=None, created_before=None):  # noqa: E501
        """OrdersList - a model defined in Swagger"""  # noqa: E501
        self._orders = None
        self._next_token = None
        self._last_updated_before = None
        self._created_before = None
        self.discriminator = None
        self.orders = orders
        if next_token is not None:
            self.next_token = next_token
        if last_updated_before is not None:
            self.last_updated_before = last_updated_before
        if created_before is not None:
            self.created_before = created_before

    @property
    def orders(self):
        """Gets the orders of this OrdersList.  # noqa: E501


        :return: The orders of this OrdersList.  # noqa: E501
        :rtype: OrderList
        """
        return self._orders

    @orders.setter
    def orders(self, orders):
        """Sets the orders of this OrdersList.


        :param orders: The orders of this OrdersList.  # noqa: E501
        :type: OrderList
        """
        if orders is None:
            raise ValueError("Invalid value for `orders`, must not be `None`")  # noqa: E501

        self._orders = orders

    @property
    def next_token(self):
        """Gets the next_token of this OrdersList.  # noqa: E501

        When present and not empty, pass this string token in the next request to return the next response page.  # noqa: E501

        :return: The next_token of this OrdersList.  # noqa: E501
        :rtype: str
        """
        return self._next_token

    @next_token.setter
    def next_token(self, next_token):
        """Sets the next_token of this OrdersList.

        When present and not empty, pass this string token in the next request to return the next response page.  # noqa: E501

        :param next_token: The next_token of this OrdersList.  # noqa: E501
        :type: str
        """

        self._next_token = next_token

    @property
    def last_updated_before(self):
        """Gets the last_updated_before of this OrdersList.  # noqa: E501

        A date used for selecting orders that were last updated before (or at) a specified time. An update is defined as any change in order status, including the creation of a new order. Includes updates made by Amazon and by the seller. All dates must be in ISO 8601 format.  # noqa: E501

        :return: The last_updated_before of this OrdersList.  # noqa: E501
        :rtype: str
        """
        return self._last_updated_before

    @last_updated_before.setter
    def last_updated_before(self, last_updated_before):
        """Sets the last_updated_before of this OrdersList.

        A date used for selecting orders that were last updated before (or at) a specified time. An update is defined as any change in order status, including the creation of a new order. Includes updates made by Amazon and by the seller. All dates must be in ISO 8601 format.  # noqa: E501

        :param last_updated_before: The last_updated_before of this OrdersList.  # noqa: E501
        :type: str
        """

        self._last_updated_before = last_updated_before

    @property
    def created_before(self):
        """Gets the created_before of this OrdersList.  # noqa: E501

        A date used for selecting orders created before (or at) a specified time. Only orders placed before the specified time are returned. The date must be in ISO 8601 format.  # noqa: E501

        :return: The created_before of this OrdersList.  # noqa: E501
        :rtype: str
        """
        return self._created_before

    @created_before.setter
    def created_before(self, created_before):
        """Sets the created_before of this OrdersList.

        A date used for selecting orders created before (or at) a specified time. Only orders placed before the specified time are returned. The date must be in ISO 8601 format.  # noqa: E501

        :param created_before: The created_before of this OrdersList.  # noqa: E501
        :type: str
        """

        self._created_before = created_before

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(OrdersList, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrdersList):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Selling Partner API for Finances

    The Selling Partner API for Finances helps you obtain financial information relevant to a seller's business. You can obtain financial events for a given order, financial event group, or date range without having to wait until a statement period closes. You can also obtain financial event groups for a given date range.  # noqa: E501

    OpenAPI spec version: v0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class FinancialEventGroup(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'financial_event_group_id': 'str',
        'processing_status': 'str',
        'fund_transfer_status': 'str',
        'original_total': 'Currency',
        'converted_total': 'Currency',
        'fund_transfer_date': 'ModelDate',
        'trace_id': 'str',
        'account_tail': 'str',
        'beginning_balance': 'Currency',
        'financial_event_group_start': 'ModelDate',
        'financial_event_group_end': 'ModelDate'
    }

    attribute_map = {
        'financial_event_group_id': 'FinancialEventGroupId',
        'processing_status': 'ProcessingStatus',
        'fund_transfer_status': 'FundTransferStatus',
        'original_total': 'OriginalTotal',
        'converted_total': 'ConvertedTotal',
        'fund_transfer_date': 'FundTransferDate',
        'trace_id': 'TraceId',
        'account_tail': 'AccountTail',
        'beginning_balance': 'BeginningBalance',
        'financial_event_group_start': 'FinancialEventGroupStart',
        'financial_event_group_end': 'FinancialEventGroupEnd'
    }

    def __init__(self, financial_event_group_id=None, processing_status=None, fund_transfer_status=None, original_total=None, converted_total=None, fund_transfer_date=None, trace_id=None, account_tail=None, beginning_balance=None, financial_event_group_start=None, financial_event_group_end=None):  # noqa: E501
        """FinancialEventGroup - a model defined in Swagger"""  # noqa: E501
        self._financial_event_group_id = None
        self._processing_status = None
        self._fund_transfer_status = None
        self._original_total = None
        self._converted_total = None
        self._fund_transfer_date = None
        self._trace_id = None
        self._account_tail = None
        self._beginning_balance = None
        self._financial_event_group_start = None
        self._financial_event_group_end = None
        self.discriminator = None
        if financial_event_group_id is not None:
            self.financial_event_group_id = financial_event_group_id
        if processing_status is not None:
            self.processing_status = processing_status
        if fund_transfer_status is not None:
            self.fund_transfer_status = fund_transfer_status
        if original_total is not None:
            self.original_total = original_total
        if converted_total is not None:
            self.converted_total = converted_total
        if fund_transfer_date is not None:
            self.fund_transfer_date = fund_transfer_date
        if trace_id is not None:
            self.trace_id = trace_id
        if account_tail is not None:
            self.account_tail = account_tail
        if beginning_balance is not None:
            self.beginning_balance = beginning_balance
        if financial_event_group_start is not None:
            self.financial_event_group_start = financial_event_group_start
        if financial_event_group_end is not None:
            self.financial_event_group_end = financial_event_group_end

    @property
    def financial_event_group_id(self):
        """Gets the financial_event_group_id of this FinancialEventGroup.  # noqa: E501

        A unique identifier for the financial event group.  # noqa: E501

        :return: The financial_event_group_id of this FinancialEventGroup.  # noqa: E501
        :rtype: str
        """
        return self._financial_event_group_id

    @financial_event_group_id.setter
    def financial_event_group_id(self, financial_event_group_id):
        """Sets the financial_event_group_id of this FinancialEventGroup.

        A unique identifier for the financial event group.  # noqa: E501

        :param financial_event_group_id: The financial_event_group_id of this FinancialEventGroup.  # noqa: E501
        :type: str
        """

        self._financial_event_group_id = financial_event_group_id

    @property
    def processing_status(self):
        """Gets the processing_status of this FinancialEventGroup.  # noqa: E501

        The processing status of the financial event group indicates whether the balance of the financial event group is settled.  Possible values:  * Open  * Closed  # noqa: E501

        :return: The processing_status of this FinancialEventGroup.  # noqa: E501
        :rtype: str
        """
        return self._processing_status

    @processing_status.setter
    def processing_status(self, processing_status):
        """Sets the processing_status of this FinancialEventGroup.

        The processing status of the financial event group indicates whether the balance of the financial event group is settled.  Possible values:  * Open  * Closed  # noqa: E501

        :param processing_status: The processing_status of this FinancialEventGroup.  # noqa: E501
        :type: str
        """

        self._processing_status = processing_status

    @property
    def fund_transfer_status(self):
        """Gets the fund_transfer_status of this FinancialEventGroup.  # noqa: E501

        The status of the fund transfer.  # noqa: E501

        :return: The fund_transfer_status of this FinancialEventGroup.  # noqa: E501
        :rtype: str
        """
        return self._fund_transfer_status

    @fund_transfer_status.setter
    def fund_transfer_status(self, fund_transfer_status):
        """Sets the fund_transfer_status of this FinancialEventGroup.

        The status of the fund transfer.  # noqa: E501

        :param fund_transfer_status: The fund_transfer_status of this FinancialEventGroup.  # noqa: E501
        :type: str
        """

        self._fund_transfer_status = fund_transfer_status

    @property
    def original_total(self):
        """Gets the original_total of this FinancialEventGroup.  # noqa: E501


        :return: The original_total of this FinancialEventGroup.  # noqa: E501
        :rtype: Currency
        """
        return self._original_total

    @original_total.setter
    def original_total(self, original_total):
        """Sets the original_total of this FinancialEventGroup.


        :param original_total: The original_total of this FinancialEventGroup.  # noqa: E501
        :type: Currency
        """

        self._original_total = original_total

    @property
    def converted_total(self):
        """Gets the converted_total of this FinancialEventGroup.  # noqa: E501


        :return: The converted_total of this FinancialEventGroup.  # noqa: E501
        :rtype: Currency
        """
        return self._converted_total

    @converted_total.setter
    def converted_total(self, converted_total):
        """Sets the converted_total of this FinancialEventGroup.


        :param converted_total: The converted_total of this FinancialEventGroup.  # noqa: E501
        :type: Currency
        """

        self._converted_total = converted_total

    @property
    def fund_transfer_date(self):
        """Gets the fund_transfer_date of this FinancialEventGroup.  # noqa: E501


        :return: The fund_transfer_date of this FinancialEventGroup.  # noqa: E501
        :rtype: ModelDate
        """
        return self._fund_transfer_date

    @fund_transfer_date.setter
    def fund_transfer_date(self, fund_transfer_date):
        """Sets the fund_transfer_date of this FinancialEventGroup.


        :param fund_transfer_date: The fund_transfer_date of this FinancialEventGroup.  # noqa: E501
        :type: ModelDate
        """

        self._fund_transfer_date = fund_transfer_date

    @property
    def trace_id(self):
        """Gets the trace_id of this FinancialEventGroup.  # noqa: E501

        The trace identifier used by sellers to look up transactions externally.  # noqa: E501

        :return: The trace_id of this FinancialEventGroup.  # noqa: E501
        :rtype: str
        """
        return self._trace_id

    @trace_id.setter
    def trace_id(self, trace_id):
        """Sets the trace_id of this FinancialEventGroup.

        The trace identifier used by sellers to look up transactions externally.  # noqa: E501

        :param trace_id: The trace_id of this FinancialEventGroup.  # noqa: E501
        :type: str
        """

        self._trace_id = trace_id

    @property
    def account_tail(self):
        """Gets the account_tail of this FinancialEventGroup.  # noqa: E501

        The account tail of the payment instrument.  # noqa: E501

        :return: The account_tail of this FinancialEventGroup.  # noqa: E501
        :rtype: str
        """
        return self._account_tail

    @account_tail.setter
    def account_tail(self, account_tail):
        """Sets the account_tail of this FinancialEventGroup.

        The account tail of the payment instrument.  # noqa: E501

        :param account_tail: The account_tail of this FinancialEventGroup.  # noqa: E501
        :type: str
        """

        self._account_tail = account_tail

    @property
    def beginning_balance(self):
        """Gets the beginning_balance of this FinancialEventGroup.  # noqa: E501


        :return: The beginning_balance of this FinancialEventGroup.  # noqa: E501
        :rtype: Currency
        """
        return self._beginning_balance

    @beginning_balance.setter
    def beginning_balance(self, beginning_balance):
        """Sets the beginning_balance of this FinancialEventGroup.


        :param beginning_balance: The beginning_balance of this FinancialEventGroup.  # noqa: E501
        :type: Currency
        """

        self._beginning_balance = beginning_balance

    @property
    def financial_event_group_start(self):
        """Gets the financial_event_group_start of this FinancialEventGroup.  # noqa: E501


        :return: The financial_event_group_start of this FinancialEventGroup.  # noqa: E501
        :rtype: ModelDate
        """
        return self._financial_event_group_start

    @financial_event_group_start.setter
    def financial_event_group_start(self, financial_event_group_start):
        """Sets the financial_event_group_start of this FinancialEventGroup.


        :param financial_event_group_start: The financial_event_group_start of this FinancialEventGroup.  # noqa: E501
        :type: ModelDate
        """

        self._financial_event_group_start = financial_event_group_start

    @property
    def financial_event_group_end(self):
        """Gets the financial_event_group_end of this FinancialEventGroup.  # noqa: E501


        :return: The financial_event_group_end of this FinancialEventGroup.  # noqa: E501
        :rtype: ModelDate
        """
        return self._financial_event_group_end

    @financial_event_group_end.setter
    def financial_event_group_end(self, financial_event_group_end):
        """Sets the financial_event_group_end of this FinancialEventGroup.


        :param financial_event_group_end: The financial_event_group_end of this FinancialEventGroup.  # noqa: E501
        :type: ModelDate
        """

        self._financial_event_group_end = financial_event_group_end

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(FinancialEventGroup, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, FinancialEventGroup):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Selling Partner API for Pricing

    The Selling Partner API for Pricing helps you programmatically retrieve product pricing and offer information for Amazon Marketplace products.  # noqa: E501

    OpenAPI spec version: v0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class OfferType(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'buying_price': 'PriceType',
        'regular_price': 'MoneyType',
        'fulfillment_channel': 'str',
        'item_condition': 'str',
        'item_sub_condition': 'str',
        'seller_sku': 'str'
    }

    attribute_map = {
        'buying_price': 'BuyingPrice',
        'regular_price': 'RegularPrice',
        'fulfillment_channel': 'FulfillmentChannel',
        'item_condition': 'ItemCondition',
        'item_sub_condition': 'ItemSubCondition',
        'seller_sku': 'SellerSKU'
    }

    def __init__(self, buying_price=None, regular_price=None, fulfillment_channel=None, item_condition=None, item_sub_condition=None, seller_sku=None):  # noqa: E501
        """OfferType - a model defined in Swagger"""  # noqa: E501
        self._buying_price = None
        self._regular_price = None
        self._fulfillment_channel = None
        self._item_condition = None
        self._item_sub_condition = None
        self._seller_sku = None
        self.discriminator = None
        self.buying_price = buying_price
        self.regular_price = regular_price
        self.fulfillment_channel = fulfillment_channel
        self.item_condition = item_condition
        self.item_sub_condition = item_sub_condition
        self.seller_sku = seller_sku

    @property
    def buying_price(self):
        """Gets the buying_price of this OfferType.  # noqa: E501


        :return: The buying_price of this OfferType.  # noqa: E501
        :rtype: PriceType
        """
        return self._buying_price

    @buying_price.setter
    def buying_price(self, buying_price):
        """Sets the buying_price of this OfferType.


        :param buying_price: The buying_price of this OfferType.  # noqa: E501
        :type: PriceType
        """
        if buying_price is None:
            raise ValueError("Invalid value for `buying_price`, must not be `None`")  # noqa: E501

        self._buying_price = buying_price

    @property
    def regular_price(self):
        """Gets the regular_price of this OfferType.  # noqa: E501


        :return: The regular_price of this OfferType.  # noqa: E501
        :rtype: MoneyType
        """
        return self._regular_price

    @regular_price.setter
    def regular_price(self, regular_price):
        """Sets the regular_price of this OfferType.


        :param regular_price: The regular_price of this OfferType.  # noqa: E501
        :type: MoneyType
        """
        if regular_price is None:
            raise ValueError("Invalid value for `regular_price`, must not be `None`")  # noqa: E501

        self._regular_price = regular_price

    @property
    def fulfillment_channel(self):
        """Gets the fulfillment_channel of this OfferType.  # noqa: E501

        The fulfillment channel for the offer listing. Possible values:  * Amazon - Fulfilled by Amazon. * Merchant - Fulfilled by the seller.  # noqa: E501

        :return: The fulfillment_channel of this OfferType.  # noqa: E501
        :rtype: str
        """
        return self._fulfillment_channel

    @fulfillment_channel.setter
    def fulfillment_channel(self, fulfillment_channel):
        """Sets the fulfillment_channel of this OfferType.

        The fulfillment channel for the offer listing. Possible values:  * Amazon - Fulfilled by Amazon. * Merchant - Fulfilled by the seller.  # noqa: E501

        :param fulfillment_channel: The fulfillment_channel of this OfferType.  # noqa: E501
        :type: str
        """
        if fulfillment_channel is None:
            raise ValueError("Invalid value for `fulfillment_channel`, must not be `None`")  # noqa: E501

        self._fulfillment_channel = fulfillment_channel

    @property
    def item_condition(self):
        """Gets the item_condition of this OfferType.  # noqa: E501

        The item condition for the offer listing. Possible values: New, Used, Collectible, Refurbished, or Club.  # noqa: E501

        :return: The item_condition of this OfferType.  # noqa: E501
        :rtype: str
        """
        return self._item_condition

    @item_condition.setter
    def item_condition(self, item_condition):
        """Sets the item_condition of this OfferType.

        The item condition for the offer listing. Possible values: New, Used, Collectible, Refurbished, or Club.  # noqa: E501

        :param item_condition: The item_condition of this OfferType.  # noqa: E501
        :type: str
        """
        if item_condition is None:
            raise ValueError("Invalid value for `item_condition`, must not be `None`")  # noqa: E501

        self._item_condition = item_condition

    @property
    def item_sub_condition(self):
        """Gets the item_sub_condition of this OfferType.  # noqa: E501

        The item subcondition for the offer listing. Possible values: New, Mint, Very Good, Good, Acceptable, Poor, Club, OEM, Warranty, Refurbished Warranty, Refurbished, Open Box, or Other.  # noqa: E501

        :return: The item_sub_condition of this OfferType.  # noqa: E501
        :rtype: str
        """
        return self._item_sub_condition

    @item_sub_condition.setter
    def item_sub_condition(self, item_sub_condition):
        """Sets the item_sub_condition of this OfferType.

        The item subcondition for the offer listing. Possible values: New, Mint, Very Good, Good, Acceptable, Poor, Club, OEM, Warranty, Refurbished Warranty, Refurbished, Open Box, or Other.  # noqa: E501

        :param item_sub_condition: The item_sub_condition of this OfferType.  # noqa: E501
        :type: str
        """
        if item_sub_condition is None:
            raise ValueError("Invalid value for `item_sub_condition`, must not be `None`")  # noqa: E501

        self._item_sub_condition = item_sub_condition

    @property
    def seller_sku(self):
        """Gets the seller_sku of this OfferType.  # noqa: E501

        The seller stock keeping unit (SKU) of the item.  # noqa: E501

        :return: The seller_sku of this OfferType.  # noqa: E501
        :rtype: str
        """
        return self._seller_sku

    @seller_sku.setter
    def seller_sku(self, seller_sku):
        """Sets the seller_sku of this OfferType.

        The seller stock keeping unit (SKU) of the item.  # noqa: E501

        :param seller_sku: The seller_sku of this OfferType.  # noqa: E501
        :type: str
        """
        if seller_sku is None:
            raise ValueError("Invalid value for `seller_sku`, must not be `None`")  # noqa: E501

        self._seller_sku = seller_sku

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(OfferType, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferType):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Selling Partner API for Finances

    The Selling Partner API for Finances helps you obtain financial information relevant to a seller's business. You can obtain financial events for a given order, financial event group, or date range without having to wait until a statement period closes. You can also obtain financial event groups for a given date range.  # noqa: E501

    OpenAPI spec version: v0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class SolutionProviderCreditEvent(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'provider_transaction_type': 'str',
        'seller_order_id': 'str',
        'marketplace_id': 'str',
        'marketplace_country_code': 'str',
        'seller_id': 'str',
        'seller_store_name': 'str',
        'provider_id': 'str',
        'provider_store_name': 'str',
        'transaction_amount': 'Currency',
        'transaction_creation_date': 'ModelDate'
    }

    attribute_map = {
        'provider_transaction_type': 'ProviderTransactionType',
        'seller_order_id': 'SellerOrderId',
        'marketplace_id': 'MarketplaceId',
        'marketplace_country_code': 'MarketplaceCountryCode',
        'seller_id': 'SellerId',
        'seller_store_name': 'SellerStoreName',
        'provider_id': 'ProviderId',
        'provider_store_name': 'ProviderStoreName',
        'transaction_amount': 'TransactionAmount',
        'transaction_creation_date': 'TransactionCreationDate'
    }

    def __init__(self, provider_transaction_type=None, seller_order_id=None, marketplace_id=None, marketplace_country_code=None, seller_id=None, seller_store_name=None, provider_id=None, provider_store_name=None, transaction_amount=None, transaction_creation_date=None):  # noqa: E501
        """SolutionProviderCreditEvent - a model defined in Swagger"""  # noqa: E501
        self._provider_transaction_type = None
        self._seller_order_id = None
        self._marketplace_id = None
        self._marketplace_country_code = None
        self._seller_id = None
        self._seller_store_name = None
        self._provider_id = None
        self._provider_store_name = None
        self._transaction_amount = None
        self._transaction_creation_date = None
        self.discriminator = None
        if provider_transaction_type is not None:
            self.provider_transaction_type = provider_transaction_type
        if seller_order_id is not None:
            self.seller_order_id = seller_order_id
        if marketplace_id is not None:
            self.marketplace_id = marketplace_id
        if marketplace_country_code is not None:
            self.marketplace_country_code = marketplace_country_code
        if seller_id is not None:
            self.seller_id = seller_id
        if seller_store_name is not None:
            self.seller_store_name = seller_store_name
        if provider_id is not None:
            self.provider_id = provider_id
        if provider_store_name is not None:
            self.provider_store_name = provider_store_name
        if transaction_amount is not None:
            self.transaction_amount = transaction_amount
        if transaction_creation_date is not None:
            self.transaction_creation_date = transaction_creation_date

    @property
    def provider_transaction_type(self):
        """Gets the provider_transaction_type of this SolutionProviderCreditEvent.  # noqa: E501

        The transaction type.  # noqa: E501

        :return: The provider_transaction_type of this SolutionProviderCreditEvent.  # noqa: E501
        :rtype: str
        """
        return self._provider_transaction_type

    @provider_transaction_type.setter
    def provider_transaction_type(self, provider_transaction_type):
        """Sets the provider_transaction_type of this SolutionProviderCreditEvent.

        The transaction type.  # noqa: E501

        :param provider_transaction_type: The provider_transaction_type of this SolutionProviderCreditEvent.  # noqa: E501
        :type: str
        """

        self._provider_transaction_type = provider_transaction_type

    @property
    def seller_order_id(self):
        """Gets the seller_order_id of this SolutionProviderCreditEvent.  # noqa: E501

        A seller-defined identifier for an order.  # noqa: E501

        :return: The seller_order_id of this SolutionProviderCreditEvent.  # noqa: E501
        :rtype: str
        """
        return self._seller_order_id

    @seller_order_id.setter
    def seller_order_id(self, seller_order_id):
        """Sets the seller_order_id of this SolutionProviderCreditEvent.

        A seller-defined identifier for an order.  # noqa: E501

        :param seller_order_id: The seller_order_id of this SolutionProviderCreditEvent.  # noqa: E501
        :type: str
        """

        self._seller_order_id = seller_order_id

    @property
    def marketplace_id(self):
        """Gets the marketplace_id of this SolutionProviderCreditEvent.  # noqa: E501

        The identifier of the marketplace where the order was placed.  # noqa: E501

        :return: The marketplace_id of this SolutionProviderCreditEvent.  # noqa: E501
        :rtype: str
        """
        return self._marketplace_id

    @marketplace_id.setter
    def marketplace_id(self, marketplace_id):
        """Sets the marketplace_id of this SolutionProviderCreditEvent.

        The identifier of the marketplace where the order was placed.  # noqa: E501

        :param marketplace_id: The marketplace_id of this SolutionProviderCreditEvent.  # noqa: E501
        :type: str
        """

        self._marketplace_id = marketplace_id

    @property
    def marketplace_country_code(self):
        """Gets the marketplace_country_code of this SolutionProviderCreditEvent.  # noqa: E501

        The two-letter country code of the country associated with the marketplace where the order was placed.  # noqa: E501

        :return: The marketplace_country_code of this SolutionProviderCreditEvent.  # noqa: E501
        :rtype: str
        """
        return self._marketplace_country_code

    @marketplace_country_code.setter
    def marketplace_country_code(self, marketplace_country_code):
        """Sets the marketplace_country_code of this SolutionProviderCreditEvent.

        The two-letter country code of the country associated with the marketplace where the order was placed.  # noqa: E501

        :param marketplace_country_code: The marketplace_country_code of this SolutionProviderCreditEvent.  # noqa: E501
        :type: str
        """

        self._marketplace_country_code = marketplace_country_code

    @property
    def seller_id(self):
        """Gets the seller_id of this SolutionProviderCreditEvent.  # noqa: E501

        The Amazon-defined identifier of the seller.  # noqa: E501

        :return: The seller_id of this SolutionProviderCreditEvent.  # noqa: E501
        :rtype: str
        """
        return self._seller_id

    @seller_id.setter
    def seller_id(self, seller_id):
        """Sets the seller_id of this SolutionProviderCreditEvent.

        The Amazon-defined identifier of the seller.  # noqa: E501

        :param seller_id: The seller_id of this SolutionProviderCreditEvent.  # noqa: E501
        :type: str
        """

        self._seller_id = seller_id

    @property
    def seller_store_name(self):
        """Gets the seller_store_name of this SolutionProviderCreditEvent.  # noqa: E501

        The store name where the payment event occurred.  # noqa: E501

        :return: The seller_store_name of this SolutionProviderCreditEvent.  # noqa: E501
        :rtype: str
        """
        return self._seller_store_name

    @seller_store_name.setter
    def seller_store_name(self, seller_store_name):
        """Sets the seller_store_name of this SolutionProviderCreditEvent.

        The store name where the payment event occurred.  # noqa: E501

        :param seller_store_name: The seller_store_name of this SolutionProviderCreditEvent.  # noqa: E501
        :type: str
        """

        self._seller_store_name = seller_store_name

    @property
    def provider_id(self):
        """Gets the provider_id of this SolutionProviderCreditEvent.  # noqa: E501

        The Amazon-defined identifier of the solution provider.  # noqa: E501

        :return: The provider_id of this SolutionProviderCreditEvent.  # noqa: E501
        :rtype: str
        """
        return self._provider_id

    @provider_id.setter
    def provider_id(self, provider_id):
        """Sets the provider_id of this SolutionProviderCreditEvent.

        The Amazon-defined identifier of the solution provider.  # noqa: E501

        :param provider_id: The provider_id of this SolutionProviderCreditEvent.  # noqa: E501
        :type: str
        """

        self._provider_id = provider_id

    @property
    def provider_store_name(self):
        """Gets the provider_store_name of this SolutionProviderCreditEvent.  # noqa: E501

        The store name where the payment event occurred.  # noqa: E501

        :return: The provider_store_name of this SolutionProviderCreditEvent.  # noqa: E501
        :rtype: str
        """
        return self._provider_store_name

    @provider_store_name.setter
    def provider_store_name(self, provider_store_name):
        """Sets the provider_store_name of this SolutionProviderCreditEvent.

        The store name where the payment event occurred.  # noqa: E501

        :param provider_store_name: The provider_store_name of this SolutionProviderCreditEvent.  # noqa: E501
        :type: str
        """

        self._provider_store_name = provider_store_name

    @property
    def transaction_amount(self):
        """Gets the transaction_amount of this SolutionProviderCreditEvent.  # noqa: E501


        :return: The transaction_amount of this SolutionProviderCreditEvent.  # noqa: E501
        :rtype: Currency
        """
        return self._transaction_amount

    @transaction_amount.setter
    def transaction_amount(self, transaction_amount):
        """Sets the transaction_amount of this SolutionProviderCreditEvent.


        :param transaction_amount: The transaction_amount of this SolutionProviderCreditEvent.  # noqa: E501
        :type: Currency
        """

        self._transaction_amount = transaction_amount

    @property
    def transaction_creation_date(self):
        """Gets the transaction_creation_date of this SolutionProviderCreditEvent.  # noqa: E501


        :return: The transaction_creation_date of this SolutionProviderCreditEvent.  # noqa: E501
        :rtype: ModelDate
        """
        return self._transaction_creation_date

    @transaction_creation_date.setter
    def transaction_creation_date(self, transaction_creation_date):
        """Sets the transaction_creation_date of this SolutionProviderCreditEvent.


        :param transaction_creation_date: The transaction_creation_date of this SolutionProviderCreditEvent.  # noqa: E501
        :type: ModelDate
        """

        self._transaction_creation_date = transaction_creation_date

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(SolutionProviderCreditEvent, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, SolutionProviderCreditEvent):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Selling Partner API for Pricing

    The Selling Partner API for Pricing helps you programmatically retrieve product pricing and offer information for Amazon Marketplace products.  # noqa: E501

    OpenAPI spec version: v0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class OfferDetail(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'my_offer': 'bool',
        'sub_condition': 'str',
        'seller_feedback_rating': 'SellerFeedbackType',
        'shipping_time': 'DetailedShippingTimeType',
        'listing_price': 'MoneyType',
        'points': 'Points',
        'shipping': 'MoneyType',
        'ships_from': 'ShipsFromType',
        'is_fulfilled_by_amazon': 'bool',
        'is_buy_box_winner': 'bool',
        'is_featured_merchant': 'bool'
    }

    attribute_map = {
        'my_offer': 'MyOffer',
        'sub_condition': 'SubCondition',
        'seller_feedback_rating': 'SellerFeedbackRating',
        'shipping_time': 'ShippingTime',
        'listing_price': 'ListingPrice',
        'points': 'Points',
        'shipping': 'Shipping',
        'ships_from': 'ShipsFrom',
        'is_fulfilled_by_amazon': 'IsFulfilledByAmazon',
        'is_buy_box_winner': 'IsBuyBoxWinner',
        'is_featured_merchant': 'IsFeaturedMerchant'
    }

    def __init__(self, my_offer=None, sub_condition=None, seller_feedback_rating=None, shipping_time=None, listing_price=None, points=None, shipping=None, ships_from=None, is_fulfilled_by_amazon=None, is_buy_box_winner=None, is_featured_merchant=None):  # noqa: E501
        """OfferDetail - a model defined in Swagger"""  # noqa: E501
        self._my_offer = None
        self._sub_condition = None
        self._seller_feedback_rating = None
        self._shipping_time = None
        self._listing_price = None
        self._points = None
        self._shipping = None
        self._ships_from = None
        self._is_fulfilled_by_amazon = None
        self._is_buy_box_winner = None
        self._is_featured_merchant = None
        self.discriminator = None
        if my_offer is not None:
            self.my_offer = my_offer
        self.sub_condition = sub_condition
        if seller_feedback_rating is not None:
            self.seller_feedback_rating = seller_feedback_rating
        self.shipping_time = shipping_time
        self.listing_price = listing_price
        if points is not None:
            self.points = points
        self.shipping = shipping
        if ships_from is not None:
            self.ships_from = ships_from
        self.is_fulfilled_by_amazon = is_fulfilled_by_amazon
        if is_buy_box_winner is not None:
            self.is_buy_box_winner = is_buy_box_winner
        if is_featured_merchant is not None:
            self.is_featured_merchant = is_featured_merchant

    @property
    def my_offer(self):
        """Gets the my_offer of this OfferDetail.  # noqa: E501

        When true, this is the seller's offer.  # noqa: E501

        :return: The my_offer of this OfferDetail.  # noqa: E501
        :rtype: bool
        """
        return self._my_offer

    @my_offer.setter
    def my_offer(self, my_offer):
        """Sets the my_offer of this OfferDetail.

        When true, this is the seller's offer.  # noqa: E501

        :param my_offer: The my_offer of this OfferDetail.  # noqa: E501
        :type: bool
        """

        self._my_offer = my_offer

    @property
    def sub_condition(self):
        """Gets the sub_condition of this OfferDetail.  # noqa: E501

        The subcondition of the item. Subcondition values: New, Mint, Very Good, Good, Acceptable, Poor, Club, OEM, Warranty, Refurbished Warranty, Refurbished, Open Box, or Other.  # noqa: E501

        :return: The sub_condition of this OfferDetail.  # noqa: E501
        :rtype: str
        """
        return self._sub_condition

    @sub_condition.setter
    def sub_condition(self, sub_condition):
        """Sets the sub_condition of this OfferDetail.

        The subcondition of the item. Subcondition values: New, Mint, Very Good, Good, Acceptable, Poor, Club, OEM, Warranty, Refurbished Warranty, Refurbished, Open Box, or Other.  # noqa: E501

        :param sub_condition: The sub_condition of this OfferDetail.  # noqa: E501
        :type: str
        """
        if sub_condition is None:
            raise ValueError("Invalid value for `sub_condition`, must not be `None`")  # noqa: E501

        self._sub_condition = sub_condition

    @property
    def seller_feedback_rating(self):
        """Gets the seller_feedback_rating of this OfferDetail.  # noqa: E501


        :return: The seller_feedback_rating of this OfferDetail.  # noqa: E501
        :rtype: SellerFeedbackType
        """
        return self._seller_feedback_rating

    @seller_feedback_rating.setter
    def seller_feedback_rating(self, seller_feedback_rating):
        """Sets the seller_feedback_rating of this OfferDetail.


        :param seller_feedback_rating: The seller_feedback_rating of this OfferDetail.  # noqa: E501
        :type: SellerFeedbackType
        """

        self._seller_feedback_rating = seller_feedback_rating

    @property
    def shipping_time(self):
        """Gets the shipping_time of this OfferDetail.  # noqa: E501


        :return: The shipping_time of this OfferDetail.  # noqa: E501
        :rtype: DetailedShippingTimeType
        """
        return self._shipping_time

    @shipping_time.setter
    def shipping_time(self, shipping_time):
        """Sets the shipping_time of this OfferDetail.


        :param shipping_time: The shipping_time of this OfferDetail.  # noqa: E501
        :type: DetailedShippingTimeType
        """
        if shipping_time is None:
            raise ValueError("Invalid value for `shipping_time`, must not be `None`")  # noqa: E501

        self._shipping_time = shipping_time

    @property
    def listing_price(self):
        """Gets the listing_price of this OfferDetail.  # noqa: E501


        :return: The listing_price of this OfferDetail.  # noqa: E501
        :rtype: MoneyType
        """
        return self._listing_price

    @listing_price.setter
    def listing_price(self, listing_price):
        """Sets the listing_price of this OfferDetail.


        :param listing_price: The listing_price of this OfferDetail.  # noqa: E501
        :type: MoneyType
        """
        if listing_price is None:
            raise ValueError("Invalid value for `listing_price`, must not be `None`")  # noqa: E501

        self._listing_price = listing_price

    @property
    def points(self):
        """Gets the points of this OfferDetail.  # noqa: E501


        :return: The points of this OfferDetail.  # noqa: E501
        :rtype: Points
        """
        return self._points

    @points.setter
    def points(self, points):
        """Sets the points of this OfferDetail.


        :param points: The points of this OfferDetail.  # noqa: E501
        :type: Points
        """

        self._points = points

    @property
    def shipping(self):
        """Gets the shipping of this OfferDetail.  # noqa: E501


        :return: The shipping of this OfferDetail.  # noqa: E501
        :rtype: MoneyType
        """
        return self._shipping

    @shipping.setter
    def shipping(self, shipping):
        """Sets the shipping of this OfferDetail.


        :param shipping: The shipping of this OfferDetail.  # noqa: E501
        :type: MoneyType
        """
        if shipping is None:
            raise ValueError("Invalid value for `shipping`, must not be `None`")  # noqa: E501

        self._shipping = shipping

    @property
    def ships_from(self):
        """Gets the ships_from of this OfferDetail.  # noqa: E501


        :return: The ships_from of this OfferDetail.  # noqa: E501
        :rtype: ShipsFromType
        """
        return self._ships_from

    @ships_from.setter
    def ships_from(self, ships_from):
        """Sets the ships_from of this OfferDetail.


        :param ships_from: The ships_from of this OfferDetail.  # noqa: E501
        :type: ShipsFromType
        """

        self._ships_from = ships_from

    @property
    def is_fulfilled_by_amazon(self):
        """Gets the is_fulfilled_by_amazon of this OfferDetail.  # noqa: E501

        When true, the offer is fulfilled by Amazon.  # noqa: E501

        :return: The is_fulfilled_by_amazon of this OfferDetail.  # noqa: E501
        :rtype: bool
        """
        return self._is_fulfilled_by_amazon

    @is_fulfilled_by_amazon.setter
    def is_fulfilled_by_amazon(self, is_fulfilled_by_amazon):
        """Sets the is_fulfilled_by_amazon of this OfferDetail.

        When true, the offer is fulfilled by Amazon.  # noqa: E501

        :param is_fulfilled_by_amazon: The is_fulfilled_by_amazon of this OfferDetail.  # noqa: E501
        :type: bool
        """
        if is_fulfilled_by_amazon is None:
            raise ValueError("Invalid value for `is_fulfilled_by_amazon`, must not be `None`")  # noqa: E501

        self._is_fulfilled_by_amazon = is_fulfilled_by_amazon

    @property
    def is_buy_box_winner(self):
        """Gets the is_buy_box_winner of this OfferDetail.  # noqa: E501

        When true, the offer is currently in the Buy Box. There can be up to two Buy Box winners at any time per ASIN, one that is eligible for Prime and one that is not eligible for Prime.  # noqa: E501

        :return: The is_buy_box_winner of this OfferDetail.  # noqa: E501
        :rtype: bool
        """
        return self._is_buy_box_winner

    @is_buy_box_winner.setter
    def is_buy_box_winner(self, is_buy_box_winner):
        """Sets the is_buy_box_winner of this OfferDetail.

        When true, the offer is currently in the Buy Box. There can be up to two Buy Box winners at any time per ASIN, one that is eligible for Prime and one that is not eligible for Prime.  # noqa: E501

        :param is_buy_box_winner: The is_buy_box_winner of this OfferDetail.  # noqa: E501
        :type: bool
        """

        self._is_buy_box_winner = is_buy_box_winner

    @property
    def is_featured_merchant(self):
        """Gets the is_featured_merchant of this OfferDetail.  # noqa: E501

        When true, the seller of the item is eligible to win the Buy Box.  # noqa: E501

        :return: The is_featured_merchant of this OfferDetail.  # noqa: E501
        :rtype: bool
        """
        return self._is_featured_merchant

    @is_featured_merchant.setter
    def is_featured_merchant(self, is_featured_merchant):
        """Sets the is_featured_merchant of this OfferDetail.

        When true, the seller of the item is eligible to win the Buy Box.  # noqa: E501

        :param is_featured_merchant: The is_featured_merchant of this OfferDetail.  # noqa: E501
        :type: bool
        """

        self._is_featured_merchant = is_featured_merchant

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(OfferDetail, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OfferDetail):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



class AccessTokenResponse:
    def __init__(self, **kwargs):
        self.access_token = kwargs.get('access_token')
        self.refresh_token = kwargs.get('refresh_token')
        self.expires_in = kwargs.get('expires')
        self.token_type = kwargs.get('token_type')



# coding: utf-8

"""
    Selling Partner API for Pricing

    The Selling Partner API for Pricing helps you programmatically retrieve product pricing and offer information for Amazon Marketplace products.  # noqa: E501

    OpenAPI spec version: v0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class GetOffersResponse(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'payload': 'GetOffersResult',
        'errors': 'ErrorList'
    }

    attribute_map = {
        'payload': 'payload',
        'errors': 'errors'
    }

    def __init__(self, payload=None, errors=None):  # noqa: E501
        """GetOffersResponse - a model defined in Swagger"""  # noqa: E501
        self._payload = None
        self._errors = None
        self.discriminator = None
        if payload is not None:
            self.payload = payload
        if errors is not None:
            self.errors = errors

    @property
    def payload(self):
        """Gets the payload of this GetOffersResponse.  # noqa: E501


        :return: The payload of this GetOffersResponse.  # noqa: E501
        :rtype: GetOffersResult
        """
        return self._payload

    @payload.setter
    def payload(self, payload):
        """Sets the payload of this GetOffersResponse.


        :param payload: The payload of this GetOffersResponse.  # noqa: E501
        :type: GetOffersResult
        """

        self._payload = payload

    @property
    def errors(self):
        """Gets the errors of this GetOffersResponse.  # noqa: E501


        :return: The errors of this GetOffersResponse.  # noqa: E501
        :rtype: ErrorList
        """
        return self._errors

    @errors.setter
    def errors(self, errors):
        """Sets the errors of this GetOffersResponse.


        :param errors: The errors of this GetOffersResponse.  # noqa: E501
        :type: ErrorList
        """

        self._errors = errors

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(GetOffersResponse, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, GetOffersResponse):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



from enum import Enum


class NotificationType(Enum):
    ANY_OFFER_CHANGED = 'ANY_OFFER_CHANGED'
    FEED_PROCESSING_FINISHED = 'FEED_PROCESSING_FINISHED'
    FBA_OUTBOUND_SHIPMENT_STATUS = 'FBA_OUTBOUND_SHIPMENT_STATUS'
    FEE_PROMOTION = 'FEE_PROMOTION'
    FULFILLMENT_ORDER_STATUS = 'FULFILLMENT_ORDER_STATUS'
    REPORT_PROCESSING_FINISHED = 'REPORT_PROCESSING_FINISHED'
    BRANDED_ITEM_CONTENT_CHANGE = 'BRANDED_ITEM_CONTENT_CHANGE'
    ITEM_PRODUCT_TYPE_CHANGE = 'ITEM_PRODUCT_TYPE_CHANGE'
    MFN_ORDER_STATUS_CHANGE = 'MFN_ORDER_STATUS_CHANGE'
    B2B_ANY_OFFER_CHANGED = 'B2B_ANY_OFFER_CHANGED'



import requests

import logging
from cachetools import TTLCache

from .credentials import Credentials
from .access_token_response import AccessTokenResponse
from sp_api.base import BaseClient

cache = TTLCache(maxsize=10, ttl=3600)
grantless_cache = TTLCache(maxsize=10, ttl=3600)

logger = logging.getLogger(__name__)


class AccessTokenClient(BaseClient):
    host = 'api.amazon.com'
    grant_type = 'refresh_token'
    path = '/auth/o2/token'

    def __init__(self, refresh_token=None):
        self.credentials = Credentials(refresh_token)

    def get_auth(self) -> AccessTokenResponse:
        """
        Get's the access token
        :return:AccessTokenResponse
        """
        global cache

        try:
            access_token = cache['access_token']
            logger.debug('from cache')
        except KeyError:
            response = requests.post(self.scheme + self.host + self.path, data=self.data, headers=self.headers)
            logger.debug('token refreshed')
            cache = TTLCache(maxsize=10, ttl=3600)
            cache['access_token'] = response.json()
            access_token = response.json()

        return AccessTokenResponse(**access_token)

    def get_grantless_auth(self):
        """
        POST /auth/o2/token HTTP/l.l
        Host: api.amazon.com
        Content-Type: application/x-www-form-urlencoded;charset=UTF-8
        grant_type=client_credentials
        &scope=sellingpartnerapi::notifications
        &client_id=foodev
        &client_secret=Y76SDl2F
        :return:
        """
        global grantless_cache
        try:
            access_token = grantless_cache['access_token']
            logger.debug('from_cache')
        except KeyError:
            response = requests.post(self.scheme + self.host + self.path, data=self.grantless_data, headers=self.headers)
            logger.debug('token_refreshed')
            grantless_cache = TTLCache(maxsize=10, ttl=3600)
            grantless_cache['access_token'] = response.json()
            access_token = response.json()

        return AccessTokenResponse(**access_token)

    @property
    def grantless_data(self):
        return {
            'grant_type': 'client_credentials',
            'client_id': self.credentials.client_id,
            'scope': 'sellingpartnerapi::notifications',
            'client_secret': self.credentials.client_secret
        }

    @property
    def data(self):
        return {
            'grant_type': self.grant_type,
            'client_id': self.credentials.client_id,
            'refresh_token': self.credentials.refresh_token,
            'client_secret': self.credentials.client_secret
        }

    @property
    def headers(self):
        return {
            'User-Agent': self.user_agent,
            'content-type': self.content_type
        }



# coding: utf-8

"""
    Selling Partner API for Pricing

    The Selling Partner API for Pricing helps you programmatically retrieve product pricing and offer information for Amazon Marketplace products.  # noqa: E501

    OpenAPI spec version: v0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six


class CompetitivePricingType(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'competitive_prices': 'CompetitivePriceList',
        'number_of_offer_listings': 'NumberOfOfferListingsList',
        'trade_in_value': 'MoneyType'
    }

    attribute_map = {
        'competitive_prices': 'CompetitivePrices',
        'number_of_offer_listings': 'NumberOfOfferListings',
        'trade_in_value': 'TradeInValue'
    }

    def __init__(self, competitive_prices=None, number_of_offer_listings=None, trade_in_value=None):  # noqa: E501
        """CompetitivePricingType - a model defined in Swagger"""  # noqa: E501
        self._competitive_prices = None
        self._number_of_offer_listings = None
        self._trade_in_value = None
        self.discriminator = None
        self.competitive_prices = competitive_prices
        self.number_of_offer_listings = number_of_offer_listings
        if trade_in_value is not None:
            self.trade_in_value = trade_in_value

    @property
    def competitive_prices(self):
        """Gets the competitive_prices of this CompetitivePricingType.  # noqa: E501


        :return: The competitive_prices of this CompetitivePricingType.  # noqa: E501
        :rtype: CompetitivePriceList
        """
        return self._competitive_prices

    @competitive_prices.setter
    def competitive_prices(self, competitive_prices):
        """Sets the competitive_prices of this CompetitivePricingType.


        :param competitive_prices: The competitive_prices of this CompetitivePricingType.  # noqa: E501
        :type: CompetitivePriceList
        """
        if competitive_prices is None:
            raise ValueError("Invalid value for `competitive_prices`, must not be `None`")  # noqa: E501

        self._competitive_prices = competitive_prices

    @property
    def number_of_offer_listings(self):
        """Gets the number_of_offer_listings of this CompetitivePricingType.  # noqa: E501


        :return: The number_of_offer_listings of this CompetitivePricingType.  # noqa: E501
        :rtype: NumberOfOfferListingsList
        """
        return self._number_of_offer_listings

    @number_of_offer_listings.setter
    def number_of_offer_listings(self, number_of_offer_listings):
        """Sets the number_of_offer_listings of this CompetitivePricingType.


        :param number_of_offer_listings: The number_of_offer_listings of this CompetitivePricingType.  # noqa: E501
        :type: NumberOfOfferListingsList
        """
        if number_of_offer_listings is None:
            raise ValueError("Invalid value for `number_of_offer_listings`, must not be `None`")  # noqa: E501

        self._number_of_offer_listings = number_of_offer_listings

    @property
    def trade_in_value(self):
        """Gets the trade_in_value of this CompetitivePricingType.  # noqa: E501


        :return: The trade_in_value of this CompetitivePricingType.  # noqa: E501
        :rtype: MoneyType
        """
        return self._trade_in_value

    @trade_in_value.setter
    def trade_in_value(self, trade_in_value):
        """Sets the trade_in_value of this CompetitivePricingType.


        :param trade_in_value: The trade_in_value of this CompetitivePricingType.  # noqa: E501
        :type: MoneyType
        """

        self._trade_in_value = trade_in_value

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(CompetitivePricingType, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CompetitivePricingType):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
